# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/


__authors__ = ["J. Garriga"]
__license__ = "MIT"
__date__ = "06/04/2020"

import numpy
from .base import Base


class NMF(Base):

    def _init_w(self):
        if self.W is None:
            Base._init_w(self)

    def _init_h(self):
        if self.H is None:
            Base._init_h(self)

    def _update_h(self):

        step = 2000

        H2 = numpy.empty(self.H.shape[0])
        h = numpy.empty(self.H.shape[1])
        for row in range(self.W.shape[1]):
            H2 = numpy.matmul(numpy.matmul(self.W.T[row], self.W), self.H) + 10**-9
            for column in range(0, self.H.shape[1], step):
                if self.indices is None:
                    h[column:column + step] = numpy.matmul(self.W.T[row], self.data[:, column:column + step])
                else:
                    h[column:column + step] = numpy.matmul(self.W.T[row], self.data[self.indices, column:column + step])
            self.H[row] *= h
            self.H[row] /= H2

    def _update_w(self):

        step = 100

        W2 = numpy.empty(self.W.shape[0])
        if self.indices is None:
            for row in range(0, len(self.W), step):
                W2 = numpy.matmul(numpy.matmul(self.W[row:row + step], self.H), self.H.T) + 10**-9
                numpy.matmul(self.data[row:row + step], self.H.T)
                self.W[row:row + step] *= numpy.matmul(self.data[row:row + step], self.H.T)
                self.W[row:row + step] /= W2
        else:
            for row in range(0, len(self.indices), step):
                indx = self.indices[row:row + step]
                W2 = numpy.matmul(numpy.matmul(self.W[row:row + step], self.H), self.H.T) + 10**-9
                numpy.matmul(self.data[indx], self.H.T)
                self.W[row:row + step] *= numpy.matmul(self.data[indx], self.H.T)
                self.W[row:row + step] /= W2

        self.W /= numpy.sqrt(numpy.sum(self.W**2.0, axis=0))

    def transform(self, H=None, W=None, max_iter=100, error_step=None, compute_w=True,
                  compute_h=True):

        self.H = H
        self.W = W

        Base.transform(self, max_iter=max_iter, error_step=error_step, compute_w=compute_w,
                       compute_h=compute_h)
